@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,7 @@
 #include <NdbOut.hpp>
 #include <WatchDog.hpp>
 #include <NdbConfig.h>
+#include <NdbSpin.h>
 
 #include <mgmapi_configuration.hpp>
 #include <kernel_config_parameters.h>
@@ -217,18 +218,21 @@ Configuration::fetch_configuration(const char* _connect_string,
   {
     Uint32 generation;
     ndb_mgm_configuration_iterator sys_iter(*p, CFG_SECTION_SYSTEM);
+    char buf[512];
+    char* sockaddr_string =
+        Ndb_combine_address_port(buf, sizeof(buf),
+        m_config_retriever->get_mgmd_host(),
+        m_config_retriever->get_mgmd_port());
+
     if (sys_iter.get(CFG_SYS_CONFIG_GENERATION, &generation))
     {
-      g_eventLogger->info("Configuration fetched from '%s:%d', unknown generation!! (likely older ndb_mgmd)",
-                          m_config_retriever->get_mgmd_host(),
-                          m_config_retriever->get_mgmd_port());
+        g_eventLogger->info("Configuration fetched from '%s', unknown generation!!"
+                            " (likely older ndb_mgmd)", sockaddr_string);
     }
     else
     {
-      g_eventLogger->info("Configuration fetched from '%s:%d', generation: %d",
-                          m_config_retriever->get_mgmd_host(),
-                          m_config_retriever->get_mgmd_port(),
-                          generation);
+        g_eventLogger->info("Configuration fetched from '%s', generation: %d",
+                            sockaddr_string, generation);
     }
   }
 
@@ -364,8 +368,17 @@ Configuration::setupConfiguration(){
   _schedulerExecutionTimer = 50;
   iter.get(CFG_DB_SCHED_EXEC_TIME, &_schedulerExecutionTimer);
 
-  _schedulerSpinTimer = 0;
+  _schedulerSpinTimer = DEFAULT_SPIN_TIME;
   iter.get(CFG_DB_SCHED_SPIN_TIME, &_schedulerSpinTimer);
+  /* Always set SchedulerSpinTimer to 0 on platforms not supporting spin */
+  if (!NdbSpin_is_supported())
+  {
+    _schedulerSpinTimer = 0;
+  }
+  g_eventLogger->info("SchedulerSpinTimer = %u", _schedulerSpinTimer);
+
+  _spinTimePerCall = 1000;
+  iter.get(CFG_DB_SPIN_TIME_PER_CALL, &_spinTimePerCall);
 
   _maxSendDelay = 0;
   iter.get(CFG_DB_MAX_SEND_DELAY, &_maxSendDelay);
@@ -394,13 +407,19 @@ Configuration::setupConfiguration(){
    */  
   if (_fsPath)
     free(_fsPath);
-  _fsPath= get_and_validate_path(iter, CFG_DB_FILESYSTEM_PATH, "FileSystemPath");
+  _fsPath= get_and_validate_path(iter,
+                                 CFG_DB_FILESYSTEM_PATH,
+                                 "FileSystemPath");
   if (_backupPath)
     free(_backupPath);
-  _backupPath= get_and_validate_path(iter, CFG_DB_BACKUP_DATADIR, "BackupDataDir");
+  _backupPath= get_and_validate_path(iter,
+                                     CFG_DB_BACKUP_DATADIR,
+                                     "BackupDataDir");
 
   if(iter.get(CFG_DB_STOP_ON_ERROR_INSERT, &m_restartOnErrorInsert)){
-    ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Invalid configuration fetched", 
+    ERROR_SET(fatal,
+              NDBD_EXIT_INVALID_CONFIG,
+              "Invalid configuration fetched", 
 	      "RestartOnErrorInsert missing");
   }
   
@@ -440,18 +459,23 @@ Configuration::setupConfiguration(){
       m_thr_config.setLockIoThreadsToCPU(maintCPU);
   }
 
-#ifdef NDB_USE_GET_ENV
-  const char * thrconfigstring = NdbEnv_GetEnv("NDB_MT_THREAD_CONFIG",
-                                               (char*)0, 0);
-#else
-  const char * thrconfigstring = NULL;
-#endif
-  if (thrconfigstring ||
-      iter.get(CFG_DB_MT_THREAD_CONFIG, &thrconfigstring) == 0)
+  const char * thrconfigstring = nullptr;
+  Uint32 mtthreads = 0;
+  Uint32 auto_thread_config = 0;
+  Uint32 num_cpus = 0;
+  iter.get(CFG_DB_AUTO_THREAD_CONFIG, &auto_thread_config);
+  iter.get(CFG_DB_NUM_CPUS, &num_cpus);
+  g_eventLogger->info("AutomaticThreadConfig = %u, NumCPUs = %u", auto_thread_config, num_cpus);
+  iter.get(CFG_DB_MT_THREADS, &mtthreads);
+  iter.get(CFG_DB_MT_THREAD_CONFIG, &thrconfigstring);
+  if (auto_thread_config == 0 &&
+      thrconfigstring != nullptr && thrconfigstring[0] != 0)
   {
     int res = m_thr_config.do_parse(thrconfigstring,
                                     _realtimeScheduler,
-                                    _schedulerSpinTimer);
+                                    _schedulerSpinTimer,
+                                    globalData.ndbRRGroups,
+                                    false);
     if (res != 0)
     {
       ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
@@ -461,46 +485,66 @@ Configuration::setupConfiguration(){
   }
   else
   {
-    Uint32 mtthreads = 0;
-    iter.get(CFG_DB_MT_THREADS, &mtthreads);
-
-    Uint32 classic = 0;
-    iter.get(CFG_NDBMT_CLASSIC, &classic);
-#ifdef NDB_USE_GET_ENV
-    const char* p = NdbEnv_GetEnv("NDB_MT_LQH", (char*)0, 0);
-    if (p != 0)
+    if (auto_thread_config != 0)
     {
-      if (strstr(p, "NOPLEASE") != 0)
-        classic = 1;
+      Uint32 num_cpus = 0;
+      iter.get(CFG_DB_NUM_CPUS, &num_cpus);
+      g_eventLogger->info("Use automatic thread configuration");
+      m_thr_config.do_parse(_realtimeScheduler,
+                            _schedulerSpinTimer,
+                            num_cpus,
+                            globalData.ndbRRGroups);
     }
-#endif
-    Uint32 lqhthreads = 0;
-    iter.get(CFG_NDBMT_LQH_THREADS, &lqhthreads);
-
-    int res = m_thr_config.do_parse(mtthreads,
-                                    lqhthreads,
-                                    classic,
-                                    _realtimeScheduler,
-                                    _schedulerSpinTimer);
-    if (res != 0)
+    else
     {
-      ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
-                "Invalid configuration fetched, invalid thread configuration",
-                m_thr_config.getErrorMessage());
+      Uint32 classic = 0;
+      iter.get(CFG_NDBMT_CLASSIC, &classic);
+#ifdef NDB_USE_GET_ENV
+      const char* p = NdbEnv_GetEnv("NDB_MT_LQH", (char*)0, 0);
+      if (p != 0)
+      {
+        if (strstr(p, "NOPLEASE") != 0)
+          classic = 1;
+      }
+#endif
+      Uint32 lqhthreads = 0;
+      iter.get(CFG_NDBMT_LQH_THREADS, &lqhthreads);
+      int res = m_thr_config.do_parse(mtthreads,
+                                      lqhthreads,
+                                      classic,
+                                      _realtimeScheduler,
+                                      _schedulerSpinTimer,
+                                      globalData.ndbRRGroups,
+                                      false);
+      if (res != 0)
+      {
+        ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
+          "Invalid configuration fetched, invalid thread configuration",
+          m_thr_config.getErrorMessage());
+      }
     }
   }
   if (NdbIsMultiThreaded())
   {
     if (thrconfigstring)
     {
-      ndbout_c("ThreadConfig: input: %s LockExecuteThreadToCPU: %s => parsed: %s",
+      ndbout_c("ThreadConfig: input: %s LockExecuteThreadToCPU: %s =>"
+               " parsed: %s",
                thrconfigstring,
                lockmask ? lockmask : "",
                m_thr_config.getConfigString());
     }
+    else if (mtthreads == 0)
+    {
+      ndbout_c("Automatic Thread Config: LockExecuteThreadToCPU: %s =>"
+               " parsed: %s",
+               lockmask ? lockmask : "",
+               m_thr_config.getConfigString());
+    }
     else
     {
-      ndbout_c("ThreadConfig (old ndb_mgmd) LockExecuteThreadToCPU: %s => parsed: %s",
+      ndbout_c("ThreadConfig (old ndb_mgmd) LockExecuteThreadToCPU: %s =>"
+               " parsed: %s",
                lockmask ? lockmask : "",
                m_thr_config.getConfigString());
     }
@@ -522,11 +566,39 @@ Configuration::setupConfiguration(){
     if (!globalData.isNdbMt)
       break;
 
+    globalData.ndbMtQueryThreads =
+      m_thr_config.getThreadCount(THRConfig::T_QUERY);
+    globalData.ndbMtRecoverThreads =
+      m_thr_config.getThreadCount(THRConfig::T_RECOVER);
     globalData.ndbMtTcThreads = m_thr_config.getThreadCount(THRConfig::T_TC);
+    globalData.ndbMtTcWorkers = globalData.ndbMtTcThreads;
+    if (globalData.ndbMtTcWorkers == 0)
+    {
+      globalData.ndbMtTcWorkers = 1;
+    }
     globalData.ndbMtSendThreads =
       m_thr_config.getThreadCount(THRConfig::T_SEND);
     globalData.ndbMtReceiveThreads =
       m_thr_config.getThreadCount(THRConfig::T_RECV);
+    /**
+     * ndbMtMainThreads is the total number of main and rep threads.
+     * There can be 0 or 1 main threads, 0 or 1 rep threads. If there
+     * is 0 main threads then the blocks handled by the main thread is
+     * handled by the receive thread and so is the rep thread blocks.
+     *
+     * When there is one main thread, then we will have both the main
+     * thread blocks and the rep thread blocks handled by this single
+     * main thread. With two main threads we will have one main thread
+     * that handles the main thread blocks and one thread handling the
+     * rep thread blocks.
+     *
+     * The nomenclature can be a bit confusing that we have a main thread
+     * that is separate from the main threads. So possibly one could have
+     * called this variable globalData.ndbMtMainRepThreads instead.
+     */
+    globalData.ndbMtMainThreads =
+      m_thr_config.getThreadCount(THRConfig::T_MAIN) +
+      m_thr_config.getThreadCount(THRConfig::T_REP);
 
     globalData.isNdbMtLqh = true;
     {
@@ -541,27 +613,44 @@ Configuration::setupConfiguration(){
 
     Uint32 threads = m_thr_config.getThreadCount(THRConfig::T_LDM);
     Uint32 workers = threads;
-    iter.get(CFG_NDBMT_LQH_WORKERS, &workers);
-
-#ifdef VM_TRACE
-#ifdef NDB_USE_GET_ENV
-    // testing
-    {
-      const char* p;
-      p = NdbEnv_GetEnv("NDBMT_LQH_WORKERS", (char*)0, 0);
-      if (p != 0)
-        workers = atoi(p);
-    }
-#endif
-#endif
-
-
-    assert(workers != 0 && workers <= MAX_NDBMT_LQH_WORKERS);
-    assert(threads != 0 && threads <= MAX_NDBMT_LQH_THREADS);
-    assert(workers % threads == 0);
+    if (threads == 0)
+      workers = 1;
 
     globalData.ndbMtLqhWorkers = workers;
     globalData.ndbMtLqhThreads = threads;
+    if (threads == 0)
+    {
+      if (!((globalData.ndbMtTcThreads == 0 &&
+             globalData.ndbMtMainThreads == 0 &&
+             globalData.ndbMtReceiveThreads == 1 &&
+             globalData.ndbMtQueryThreads == 0) ||
+            (globalData.ndbMtTcThreads == 0 &&
+             globalData.ndbMtMainThreads == 1 &&
+             globalData.ndbMtReceiveThreads == 1 &&
+             globalData.ndbMtQueryThreads == 0)))
+      {
+        ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
+                  "Invalid configuration fetched. ",
+                  "Setting number of ldm threads to 0 must be combined"
+                  " with 0 query, tc, rep thread and 0/1 main thread"
+                  " and 1 recv thread");
+      }
+    }
+    Uint32 query_threads_per_ldm = globalData.ndbMtQueryThreads / workers;
+    if (workers * query_threads_per_ldm != globalData.ndbMtQueryThreads)
+    {
+      ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
+                "Invalid configuration fetched. ",
+                "Number of query threads must be a multiple of the number"
+                " of LDM threads.");
+    }
+    globalData.QueryThreadsPerLdm = query_threads_per_ldm;
+    if (globalData.ndbMtRecoverThreads > MAX_NDBMT_QUERY_THREADS)
+    {
+      ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
+                "Invalid configuration fetched. ",
+                "Sum of recover threads and query threads can be max 127");
+    }
   } while (0);
 
   calcSizeAlt(cf);
@@ -585,6 +674,11 @@ Configuration::schedulerExecutionTimer(int value) {
     _schedulerExecutionTimer = value;
 }
 
+Uint32
+Configuration::spinTimePerCall() const {
+  return _spinTimePerCall;
+}
+
 int 
 Configuration::schedulerSpinTimer() const {
   return _schedulerSpinTimer;
@@ -1343,7 +1437,8 @@ Configuration::setLockCPU(NdbThread * pThread,
   }
   else if (!NdbIsMultiThreaded())
   {
-    BlockNumber list[] = { DBDIH };
+    BlockNumber list[1];
+    list[0] = numberToBlock(TRPMAN, 1);
     res = m_thr_config.do_bind(pThread, list, 1);
   }
 
@@ -1396,7 +1491,8 @@ Configuration::setThreadPrio(NdbThread * pThread,
   }
   else if (!NdbIsMultiThreaded())
   {
-    BlockNumber list[] = { DBDIH };
+    BlockNumber list[1];
+    list[0] = numberToBlock(TRPMAN, 1);
     res = m_thr_config.do_thread_prio(pThread, list, 1, thread_prio);
   }
 
